/*
 *
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package net.srt.flink.client.cdc.doris;

import java.io.Serializable;
import java.util.Map;

/**
 * @author BillyXing
 * @since 2022/11/10 19:37
 */
public class AdditionalColumnEntry<K, V> implements Map.Entry<K, V>, Serializable {

    private static final long serialVersionUID = 45678994131L;

    private final K k;
    private final V v;

    private AdditionalColumnEntry(K k, V v) {
        this.k = k;
        this.v = v;
    }

    public static <K, V> AdditionalColumnEntry<K, V> of(K k, V v) {
        return new AdditionalColumnEntry<K, V>(k, v);
    }

    @Override
    public K getKey() {
        return k;
    }

    @Override
    public V getValue() {
        return v;
    }

    @Override
    public V setValue(V value) {
        throw new UnsupportedOperationException();
    }
}
